package xyz.zhouxy.plusone.commons.net;

import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

import com.google.common.annotations.Beta;

@Beta
public class MoreInetAddresses {

    public static NetworkInterfaceInfo getNetworkInterfaceInfo() throws SocketException {
        final Enumeration<NetworkInterface> all = NetworkInterface.getNetworkInterfaces();
        NetworkInterface networkInterface;
        while (all.hasMoreElements()) {
            networkInterface = all.nextElement();
            if (networkInterface.isLoopback() || networkInterface.isVirtual()) {
                continue;
            }
            final byte[] mac = networkInterface.getHardwareAddress();
            if (mac != null) {
                return new NetworkInterfaceInfo(networkInterface, mac);
            }
        }
        throw new IllegalStateException("No available network interface found");
    }

    public static Inet4Address getIpv4(NetworkInterface networkInterface) {
        final Enumeration<InetAddress> ips = networkInterface.getInetAddresses();
        InetAddress ip;
        while (ips.hasMoreElements()) {
            ip = ips.nextElement();
            if (!ip.isLoopbackAddress() && (ip instanceof Inet4Address)) {
                return (Inet4Address) ip;
            }
        }
        throw new IllegalStateException("No available address found");
    }

    public static Inet6Address getIpv6(NetworkInterface networkInterface) {
        final Enumeration<InetAddress> ips = networkInterface.getInetAddresses();
        InetAddress ip;
        while (ips.hasMoreElements()) {
            ip = ips.nextElement();
            if (!ip.isLoopbackAddress() && (ip instanceof Inet6Address)) {
                return (Inet6Address) ip;
            }
        }
        throw new IllegalStateException("No available address found");
    }

    private MoreInetAddresses() {
        throw new UnsupportedOperationException("Utility class");
    }
}
